'use strict';

var cameraChoixController = [
    '$scope',
    '$route',
    '$translate',
    'GeneanetDevice',
    'ProjetService',
    'GeneanetFile',
    'GeneanetImage',
    function($scope, $route, $translate, GeneanetDevice, ProjetService, GeneanetFile, GeneanetImage) {
        document.addEventListener('deviceready', GeneanetDevice.determineOrientationCurrentPage);

        var currentProjet = ProjetService.currentProjet;
        var currentDepot = ProjetService.currentDepot;
        var currentPhoto = ProjetService.currentPhoto;


        // On génère de quoi afficher le select de sections.
        var sectionValueNotSelected = -1;
        var sectionValueUnknown = 0;

        // On liste les sections disponibles pour le select.
        var sections = [];
        if (currentProjet.sections.length) {
            var sectionNotSelected = {id: sectionValueNotSelected, nom: $translate.instant('camerapage_choix.section_choice')};
            sections.push(sectionNotSelected);

            var sectionUnknown = {id: sectionValueUnknown, nom: $translate.instant('camerapage_choix.section_unknown')};
            sections.push(sectionUnknown);

            for (var i = 0; i < currentProjet.sections.length; i++) {
                sections.push(currentProjet.sections[i]);
            }
        }
        $scope.sections = sections;

        // On va déterminer la section actuellement selectionnée.
        var currentSectionId = sectionValueNotSelected;
        if (currentDepot) {
            currentSectionId = currentDepot.sectionId ? currentDepot.sectionId : sectionValueUnknown;
        }
        for (var i = 0; i < sections.length; i++) {
            if (sections[i].id == currentSectionId) {
                $scope.currentSection = sections[i];
                break;
            }
        }

        $scope.photoId = null;
        $scope.cadreClass = '';

        // On ajoute les actions sur les liens.
        // Permet de voir toutes les photos du projet.
        $scope.seeAllPictures = function () {
            ProjetService.redirectToGallery();
        };

        // Callback sur le focus de section pour désactiver la première option.
        $scope.focusSection = function () {
            angular.element('#sections select option[value="'+sectionValueNotSelected+'"]').prop('disabled', true);
        };

        // Callback sur le changement de section.
        $scope.changeSection = function () {
            nextGrave();
        };
        // Permet de passer à la tombe suivante.
        $scope.nextGrave = nextGrave;
        // Permet d'ajouter une nouvelle vue.
        $scope.nextView = addPhoto;

        if (!currentPhoto) {
            // Pas de photo courante.
            // Le projet n'a pas de section ou bien une section est selectionnée, on démarre directement l'appareil photo.
            if (currentProjet.sections.length == 0 || $scope.currentSection.id != sectionValueNotSelected) {
                addPhoto();
            }
        } else {
            // Si non, on affiche juste les infos sur l'image courante.
            refreshPhotoInfos();
        }

        /**
         * Démarre la prise de photo.
         */
        function addPhoto() {
            ProjetService.currentPhoto = null;

            // On récupère l'id de la section. La valeur dans scope est pourrie, elle ne se met jamais à jour (peut etre dû à un bug navigateur)!
            var sectionId = angular.element('#sections select option:selected').val();
            sectionId = parseInt(sectionId);
            sectionId = sectionId <= 0 ? null : sectionId;

            ProjetService.addPhoto(sectionId)
                .then(function (photo) {
                    currentPhoto = photo;
                    $scope.$apply(refreshPhotoInfos);
                })
                .catch(function (err) {
                    console.error(err);
                    window.history.back();
                });
        }

        /**
         * Permet de passer à la tombe suivante.
         */
        function nextGrave() {
            ProjetService.currentDepot = null;
            addPhoto();
        }

        /**
         * Actualise l'affichage avec la photo courante.
         */
        function refreshPhotoInfos() {
            // On a déjà fait une photo, on affiche la page pour l'accès aux différentes actions.
            $scope.photoId = currentPhoto.id;
            $scope.imgSrc = currentProjet.getFirstPhotoUrl();

            $scope.nameProjet = currentProjet.nom;
            $scope.namePlace = currentProjet.place;
            $scope.countPictures = currentProjet.countPhotos();

            if ($scope.countPictures == 1) {
                $scope.cadreClass = 'cadre1';
            } else if ($scope.countPictures == 2) {
                $scope.cadreClass = 'cadre2';
            } else if ($scope.countPictures > 2) {
                $scope.cadreClass = 'cadre3';
            }
        }
    }
];