'use strict';

var configurationController = [
    '$scope',
    '$route',
    '$translate',
    'GeneanetDevice',
    'GeneanetOverlay',
    'ProjetService',
    'GeneanetFile',
    function($scope, $route, $translate, GeneanetDevice, GeneanetOverlay, ProjetService, GeneanetFile) {
        document.addEventListener('deviceready', GeneanetDevice.determineOrientationCurrentPage);

        $scope.back = function () {
            window.history.back();
        };

        $scope.$on('overlayDialogResponse', function(event, data) {
            if (data.response) {
                var photos = [];
                var projetsToUpdate = {};

                for (var i = 0; i < ProjetService.projets.length; i++) {
                    var projet = ProjetService.projets[i];
                    for (var j = 0; j < projet.depots.length; j++) {
                        var depot = projet.depots[j];
                        // On clone l'array de photos sinon pendant la suppression, le tableau bouge et on en rate.
                        var photosCloned = depot.photos.slice(0);
                        for (var k = 0; k < photosCloned.length; k++) {
                            var photo = photosCloned[k];
                            if (photo.isSync) {
                                photos.push(photo);
                                depot.deletePhoto(photo);
                                projetsToUpdate[projet.id] = projet;
                            }
                        }
                    }
                }

                if (photos.length > 0) {
                    GeneanetOverlay.openProgressBar($translate.instant('alert.delete_pictures_progress'), 0, photos.length);

                    var promiseFiles = new Promise(function (resolve) {
                        var promisesDelete = [];
                        for (var i = 0; i < photos.length; i++) {
                            var promise = GeneanetFile.deleteFileFromUrl(photos[i].nativeUrl);
                            promise.finally(function () {
                                GeneanetOverlay.advance();
                            });
                            promisesDelete.push(promise);
                        }
                        Promise.all(promisesDelete).finally(function () {
                            // Tous les fichiers sont supprimés (ou échoués mais on compte comme supprimé).
                            resolve();
                        });
                    });

                    promiseFiles.then(function () {
                        // on nettoie les dépots vides et on met à jour les projets impactés.
                        var projetsUpdatedPromises = [];
                        for (var projetId in projetsToUpdate) {
                            // On supprime les depots vides.
                            projetsToUpdate[projetId].deleteEmptyDepots();
                            // On met à jour le storage.
                            projetsUpdatedPromises.push(ProjetService.updateProjet(projetsToUpdate[projetId]));
                        }

                        Promise.all(projetsUpdatedPromises).then(function () {
                            $scope.$on('GeneanetOverlayClose', function() {
                                $route.reload();
                            });
                            GeneanetOverlay.openOverlay(
                                $translate.instant('alert.delete_pictures_synchronized_done'),
                                $translate.instant('alert.delete_pictures_synchronized_count')+' '+photos.length
                            );
                        });
                    });
                }
            }
        });

        $scope.deletePicturesSynchronized = function() {
            GeneanetOverlay.openDialog(
                $translate.instant('alert.delete_pictures_synchronized'),
                $translate.instant('dialog.yes'),
                'btn btn-gnn-primary',
                $translate.instant('dialog.no'),
                'btn btn-gnn-secondary'
            );
        };
    }
]