'use strict';

var galleryProjectController = [
    '$scope',
    '$rootScope',
    '$location',
    '$route',
    '$routeParams',
    '$translate',
    '$compile',
    '$timeout',
    'GeneanetDevice',
    'GeneanetImage',
    'GeneanetFile',
    'GeneanetOverlay',
    'ProjetService',
    function ($scope, $rootScope, $location, $route, $routeParams, $translate, $compile, $timeout, GeneanetDevice, GeneanetImage, GeneanetFile, GeneanetOverlay, ProjetService) {
        // Constante du nom d'évent qui signale qu'on démarre une action delete.
        var NAME_EVENT_ACTION_DELETE = "action_delete";
        // permet de stocker le template des block d'image pour eviter de le regénérer à chaque fois.
        var templatePicture = null;
        // Permet de savoir d'où démarrer le limit SQL pour la récupération des depots.
        var startLimit = 0;
        // Constante permettant de limiter la récupération d'images par lot de 18.
        var MAX_LIMIT = 18;
        // permet de préciser si le projet est vide
        $scope.projectEmpty = null;

        document.addEventListener('deviceready', GeneanetDevice.determineOrientationCurrentPage);

        // Un manager d'action permettant de stopper le chargement des images ou d'agir une fois qu'elles sont chargées.
        var actionsManager = new GeneanetActionsManager();

        $scope.back = function () {
            actionsManager.stopAll();
            window.history.back();
        };

        $scope.$on('resize', function () {
            // Sur un resize (changement d'orientation de l'écran), on stoppe tout.
            actionsManager.stopAll().then(function () {
                // On supprime les éléments actuellement présent, on remonte en haut et on recommence au début.
                angular.element("#content-gallery li").each(function() {
                    angular.element(this).remove();
                    if (angular.element("#content-gallery li").length <= 0) {
                        angular.element(".content-cell").scrollTop(0);
                        prevHeight = 0;
                        startLimit = 0;

                        $scope.cancelActionToDeleteDepot();
                        $timeout(showNextDepotsGroup, 0, true);

                        $scope.stopScrollInfinite = false;
                    }
                });
            });
        });

        // On récupère le projet actuellement affiché.
        var projet = ProjetService.currentProjet;
        $scope.nameProject = projet.nom;

        // On configure les actions sur les boutons du bas.
        // On configure l'action pour démarrer la prise de photo.
        $scope.takePicture = function () {
            actionsManager.stopAll();

            ProjetService.currentDepot = null;
            ProjetService.currentPhoto = null;

            $location.path('/cameraChoix/');
        };

        // On configure les event, etc pour gérer la suppression de depots.
        $scope.startActionToDeleteDepot = function() {
            $scope.lanchSup = true;
            $rootScope.$broadcast(NAME_EVENT_ACTION_DELETE, {start: true});
        }

        $scope.cancelActionToDeleteDepot = function() {
            $scope.lanchSup = false;
            $rootScope.$broadcast(NAME_EVENT_ACTION_DELETE, {start: false});
        }

        $scope.suppressionDepot = function () {
            var depots = [];
            angular.element('#content-gallery :checked').each(function () {
                depots.push(angular.element(this).scope().depot);
            });

            if (depots.length == 0) {
                $scope.cancelActionToDeleteDepot();

                return;
            }

            var totalPhotos = 0;
            for (var i = 0; i < depots.length; i++) {
                totalPhotos += depots[i].countPhotos();
            }
            GeneanetOverlay.openProgressBar($translate.instant('progress_bar.delete_picture'), 0, totalPhotos);

            var promisesDelete = [];
            for (var i = 0; i < depots.length; i++) {
                var depot = depots[i];
                projet.deleteDepot(depot);
                for (var j = 0; j < depot.countPhotos(); j++) {
                    var promise = GeneanetFile.deleteFileFromUrl(depot.photos[j].nativeUrl);
                    promise.finally(function () {
                        GeneanetOverlay.advance();
                    });
                    promisesDelete.push(promise);
                }
            }
            Promise.all(promisesDelete).finally(function () {
                // Tous les fichiers sont supprimés (ou échoués mais on compte comme supprimé).
                ProjetService.updateProjet(projet).then(function () {
                    GeneanetOverlay.close();
                    $route.reload();
                });
            });
        };

        // Config pour le scroll infini.
        var prevHeight = 0;
        $scope.stopScrollInfinite = false;
        $scope.childScrollInfinite = angular.element("#content-gallery");
        $scope.distanceScrollInfinite = function() {
            return angular.element("#content-gallery").children(":first-child").height() * 2;
        }
        $scope.functScrollInfinite = function() {
            if (angular.element("#content-gallery").height() > prevHeight) {
                prevHeight = angular.element("#content-gallery").height();
                $scope.cancelActionToDeleteDepot();
                showNextDepotsGroup();
            }
        };

        $scope.openSettings = function() {
            var descriptionOverlaySettings = angular.element("<div>");
            if (projet.isPrivate()) {
                descriptionOverlaySettings.append(
                    angular
                        .element("<a>")
                        .attr("href", "#/createProject/")
                        .text($translate.instant("create_projectpage.edit_project"))
                        .addClass("btn btn-gnn-secondary")
                        .css("width", "100%")
                        .css("margin-bottom", "1.5rem")
                );
            }
            descriptionOverlaySettings.append(
                angular
                    .element("<a>")
                    .addClass("btn btn-gnn-secondary")
                    .text($translate.instant("gallery.delete_project"))
                    .css("width", "100%")
                    .click(function(event) {
                        event.preventDefault();

                        $scope.$on("overlayDialogResponse", function(event, data) {
                            if (data.response) {
                                deleteProject();
                            } else {
                                $route.reload();
                            }
                        });
                        GeneanetOverlay.openDialog(
                            $translate.instant("gallery.delete_query"),
                            $translate.instant("dialog.yes"),
                            "btn btn-gnn-primary",
                            $translate.instant("dialog.no"),
                            "btn btn-gnn-secondary"
                        );
                    })
            );

            if (projet.idProjetGnn !== null) {
                descriptionOverlaySettings.append(
                    angular
                        .element("<a>")
                        .attr("href", "#/detailsProject/"+projet.idProjetGnn)
                        .text($translate.instant("details.title"))
                        .addClass("btn btn-gnn-secondary")
                        .css("width", "100%")
                        .css("margin-top", "1.5rem")
                );
            }

            GeneanetOverlay.openOverlay(" ", descriptionOverlaySettings);
        };

        // On charge le premier lot de depots a afficher.
        // On passe par timeout si non lors d'un delete de depot, ça va trop vite et le cycle angular n'a pas fini de charger.
        $timeout(showNextDepotsGroup, 0, true);

        /**
         * Récupère les prochains depots à afficher, les infos associées et les affiche.
         */
        function showNextDepotsGroup() {
            $scope.generateListFinished = false;

            var inverseOrderBy = $routeParams.inverseOrderBy == "true" ? true : false;
            var depots = projet.getDepotsGroup(startLimit, MAX_LIMIT, inverseOrderBy);
            startLimit = startLimit + MAX_LIMIT;

            // si on a récupérer moins d'image que la taille du limit, on stoppe le calcul du scroll infini.
            if (depots.length < MAX_LIMIT) {
                $scope.stopScrollInfinite = true;
            }

            // première fois, on va setter le premier lot de dépot.
            if ($scope.projectEmpty == null) {
                $scope.projectEmpty = depots.length <= 0 ? true : false;
            }
            // on a fini de générer les blocs.
            $scope.generateListFinished = true;

            var generateListAction = actionsManager.createAction();
            // on parcourt les images une à une et on les ajoute à la page.
            for (var i = 0; i < depots.length; i++) {
                if (generateListAction.canContinue()) {
                    appendDepot(depots[i]);
                    if (i+1 == depots.length) {
                        // C'est la fin.
                        generateListAction.done();
                    }
                } else {
                    break;
                }
            }

            generateListAction.promise.then(function () {
                // On a généré l'ensemble des éléments, on va pouvoir afficher les images.
                var showDepotAction = actionsManager.createAction({needUpdate: false});
                showDepotAction.promise.finally(function () {
                    if (showDepotAction.data.needUpdate) {
                        ProjetService.updateProjet(projet);
                    }

                    if ($('#content-gallery').height() <= $('#content-gallery').parent().height()) {
                        // Si jamais les images font piles la taille de l'écran, on n'a pas de scroll.
                        // On lance donc la méthode de scroll infini pour éviter de bloquer le scroll.
                        $scope.functScrollInfinite();
                    }
                });
                showDepotPhoto(depots, showDepotAction);
            });
        }

        /**
         * Ajoute le depot dans le dom.
         *
         * @param {Depot} depot
         */
        function appendDepot(depot) {
            if (templatePicture == null) {
                var li = angular.element("<li>");
                var label = angular.element("<label>");
                var input = angular.element("<input>");
                var link = angular.element("<a>");
                var spanPicture = angular.element("<span>");
                var img = angular.element("<img>");

                input.addClass("checkbox-gallery")
                    .attr("type", "checkbox");

                label.addClass("label-gallery")
                    .attr("ng-if", "launchSup");
                label.append(input);
                li.append(label);

                img.attr("id", "depot-{{depot.id}}")
                    .attr("src", "img/spinner/spinner-black.gif");

                spanPicture.addClass("content-picture");
                spanPicture.append(img);
                link.append(spanPicture);

                var spanCountPictures = angular.element("<span>");
                spanCountPictures.addClass("notif-nb-img fs-big")
                    .attr("ng-if", "depot.countPhotos() > 1")
                    .html("{{ depot.countPhotos() }}");
                link.append(spanCountPictures);

                link.attr("ng-click", "selectedDepot(depot)");
                link.attr("class", "{{ classCss }}");

                li.append(link);

                templatePicture = $compile(li);
            }

            var scopeDepot = $rootScope.$new();
            scopeDepot.depot = depot;
            var nbPhotos = depot.countPhotos();
            if (nbPhotos == 1) {
                scopeDepot.classCss = "cadre1";
            } else if (nbPhotos == 2) {
                scopeDepot.classCss = "cadre2";
            } else if (nbPhotos > 2) {
                scopeDepot.classCss = "cadre3";
            } else {
                // Depot vide, on l'affiche pas.
                return;
            }
            scopeDepot.selectedDepot = selectedDepot;
            scopeDepot.$on(NAME_EVENT_ACTION_DELETE, function(event, data) {
                scopeDepot.launchSup = data.start;
            });

            $scope.$apply(function () {
                templatePicture(scopeDepot, function(cloneElement, scope) {
                    angular.element("#content-gallery").append(cloneElement);
                });
            });
        }

        /**
         * Action lorsque l'on sélectionne un depot.
         *
         * @param {Depot} depot
         */
        function selectedDepot(depot) {
            actionsManager.stopAll();
            ProjetService.currentDepot = depot;
            $location.path('/pictureReader/');
        }

        /**
         * Méthode appelée récursivement pour charger une à une les images de chaque depot.
         *
         * @param {Array} depots
         * @param {Action} showDepotAction
         */
        function showDepotPhoto(depots, showDepotAction) {
            if (depots.length == 0) {
                // On a tout chargé.
                showDepotAction.done();
                return;
            }
            if (!showDepotAction.canContinue()) {
                console.log("Chargement des images arrété.");
                return;
            }


            var depot = depots.shift();
            var imgEl = angular.element("#depot-"+depot.id);
            if (imgEl.length == 0) {
                // L'image n'a pas été trouvée dans le DOM.
                console.error("L'element img n'est pas dans le dom.");
                return;
            }
            imgEl = imgEl[0];

            // On détermine l'orientation du téléphone.
            var screenOrientationPortrait = window.matchMedia("(orientation: portrait)").matches;

            // On récupère le base64 approprié pour l'orientation.
            var base64Picture;
            if (screenOrientationPortrait) {
                base64Picture = depot.base64Portrait;
            } else {
                base64Picture = depot.base64Landscape;
            }

            // Si l'image échoue au chargement, on rebascule sur l'url native en espérant que c'est le base64 qui est foieux et on va générer de nouveau le base64.
            imgEl.onerror = function() {
                base64Picture = false;
                imgEl.onerror = null;
                imgEl.src = depot.getFirstPhotoUrl();
            }

            imgEl.onload = function () {
                // Au chargement de l'image, si c'est l'image native qui est affichée, on va générer le base64.
                if (!base64Picture) {
                    var newWidth = angular.element(".content-picture").width();
                    var newHeight = angular.element(".content-picture").height();
                    base64Picture = GeneanetImage.resize(imgEl, newWidth, newHeight, "image/jpg");

                    if (screenOrientationPortrait) {
                        depot.base64Portrait = base64Picture;
                    } else {
                        depot.base64Landscape = base64Picture;
                    }
                    showDepotAction.data.needUpdate = true;
                    // On remplace l'image par le base64 (on vire le onload pour éviter de boucler).
                    imgEl.onload = null;
                    imgEl.src = base64Picture;
                }
                // Et on déclenchement le chargement de la seconde image.
                showDepotPhoto(depots, showDepotAction);
            };

            if (!base64Picture) {
                imgEl.src = depot.getFirstPhotoUrl();
            } else {
                imgEl.src = base64Picture;
            }
        }

        /**
         * Fonction permettant de supprimer le projet.
         *
         * @return {Promise}
         */
        function deleteProject() {
            var promiseDelete = new Promise(function (resolve) {
                var nbPhotos = projet.countPhotos();
                if (nbPhotos) {
                    var promisesDelete = [];
                    for (var i = 0; i < projet.depots.length; i++) {
                        var depot = projet.depots[i];
                        for (var j = 0; j < depot.countPhotos(); j++) {
                            var promise = GeneanetFile.deleteFileFromUrl(depot.photos[j].nativeUrl);
                            promise.finally(function () {
                                GeneanetOverlay.advance();
                            });
                            promisesDelete.push(promise);
                        }
                    }
                    Promise.all(promisesDelete).finally(function () {
                        resolve();
                    });
                } else {
                    resolve();
                }
            });
            promiseDelete.then(function () {
                // Tout est supprimé. On supprime le projet et on back.
                ProjetService.deleteProjet(projet).then(function () {
                    $scope.back();
                });
            });
        }
    }
];
