'use strict';

var listProjectGnnController = [
    '$scope',
    '$http',
    '$translate',
    '$location',
    '$timeout',
    'GeneanetDevice',
    'GeneanetOverlay',
    'GeneanetGeolocation',
    'urlsWebservice',
    'ProjetService',
    function ($scope, $http, $translate, $location, $timeout, GeneanetDevice, GeneanetOverlay, GeneanetGeolocation, urlsWebservice, ProjetService) {
        document.addEventListener('deviceready', GeneanetDevice.determineOrientationCurrentPage);

        $scope.retryLocalisation = false;
        $scope.template_list = {url: 'templates/list_project.html'};
        $scope.back = function () {
            window.history.back();
        };

        var listerProjectGnn = function () {
            var loadProject = function(projets) {
                if (projets.length == 0) {
                    $scope.noProjects = true;
                } else {
                    var projetsInfos = [];
                    for (var i = 0; i < projets.length; i++) {
                        var projet = new Projet();
                        projet.idProjetGnn = projets[i].id;
                        projet.nom = projets[i].nom_projet;
                        projet.type = projets[i].type_projet;
                        projet.place = projets[i].place;
                        projet.sections = projets[i].sections;

                        var elementsGeo = [];
                        if (projets[i].lat != null && projets[i].lon != null) {
                            elementsGeo.push(projets[i].lat);
                            elementsGeo.push(projets[i].lon);
                        } else {
                            if (projets[i].country != "") {
                                elementsGeo.push(projets[i].country);
                            }
                            if (projets[i].region != "") {
                                elementsGeo.push(projets[i].region);
                            }
                            if (projets[i].subregion != "") {
                                elementsGeo.push(projets[i].subregion);
                            }
                        }
                        projet.setGeo(elementsGeo);

                        projetsInfos.push({projet: projet, nbPhotosText: ''});

                    }
                    $scope.displayDetails = true;
                    $scope.projetsInfos = projetsInfos;
                }

                // on ferme l'overlay une fois le apply terminé.
                $timeout(GeneanetOverlay.close, 0, true);

            }

            // on va vérifier si on a du cache et qu'il est valide.
            var cacheAvailable = false;
            if (sessionStorage.getItem("projectsGnnCache")) {
                var projectsGnnCache = JSON.parse(sessionStorage.getItem("projectsGnnCache"));
                var timeCache = projectsGnnCache.time ? new Date(projectsGnnCache.time) : null;

                if (
                    timeCache &&
                    new Date(timeCache).setMinutes(timeCache.getMinutes()+5) >= new Date() &&
                    projectsGnnCache.projects
                ) {
                    cacheAvailable = true;
                }
            }

            if (cacheAvailable) {
                loadProject(JSON.parse(sessionStorage.getItem("projectsGnnCache")).projects);
            } else {
                GeneanetGeolocation.getCurrentPosition(60000, 4000, true)
                    .then(function (position) {
                        // Si on a réussi à localiser, on n'a plus besoin du bouton pour réessayer.
                        $scope.retryLocalisation = false;

                        var paramsHttp = {params: {
                            size: 50,
                            lat: position.coords.latitude,
                            lon: position.coords.longitude,
                            range: 20,
                        }};

                        $http
                            .get(urlsWebservice.galleryUrl + 'projects/public', paramsHttp)
                            .success(
                                function (data) {
                                    var projectsGnnCache = {
                                        time: new Date(),
                                        projects: data.projects
                                    }
                                    sessionStorage.setItem("projectsGnnCache", JSON.stringify(projectsGnnCache));
                                    loadProject(data.projects);
                                }
                            )
                            .error(
                                function(data, status, headers, config) {
                                    var title = $translate.instant('alert.error');
                                    var content = $translate.instant('alert.error_get_projectGnn');
                                    GeneanetOverlay.openOverlay(title, content);
                                }
                            );
                    })
                    .catch(function () {
                        // Si on n'arrive pas à localiser l'utilisateur,
                        // on va lui donner une autre chance de se localiser avec un texte+bouton.
                        // Si jamais on y arrive toujours pas, on retire le bouton et donc il ne pourra plus faire l'essai.
                        $scope.retryLocalisation = !$scope.retryLocalisation;
                        // on ferme l'overlay.
                        $timeout(GeneanetOverlay.close, 0, true);
                    });
            }
        };

        $scope.selected = function(projet) {
            ProjetService.insertProjet(projet).then(function () {
                $scope.back();
            });
        };

        $scope.details = function(projet) {
            $location.path("/detailsProject/"+projet.idProjetGnn);
        };

        $scope.createProject = function() {
            ProjetService.resetContext();

            $location.path('/createProject/');
        };

        $scope.retryLocalisationAction = function() {
            GeneanetOverlay.openSpinner($translate.instant('spinner.get_data'));
            listerProjectGnn();
        };

        GeneanetOverlay.openSpinner($translate.instant('spinner.get_data'));
        // on decale le chargement des projets d'une seconde après la fin de l'$apply pour que la vue s'affiche avec le spinner avant de "freeze".
        $timeout(function () {
            //Afficher la liste
            listerProjectGnn();
            $scope.template_list = {
                url: 'templates/list_project.html'
            };
        }, 1000, true);
    }
];