'use strict';

var pictureReaderController = [
    '$route',
    '$location',
    '$scope',
    '$routeParams',
    'GeneanetDevice',
    'GeneanetFile',
    'ProjetService',
    function($route, $location, $scope, $routeParams, GeneanetDevice, GeneanetFile, ProjetService) {
        document.addEventListener('deviceready', GeneanetDevice.determineOrientationCurrentPage);

        var CURRENT_ACTION_DELETE_PICTURE = "delete_picture";
        var CURRENT_ACTION_UNLINK_PICTURE = "unlink_picture";
        var currentAction = null;

        if (ProjetService.currentDepot == null) {
            window.history.back();
        }

        $scope.back = function () {
            window.history.back();
        };

        // L'image courante.
        $scope.currentPositionPicture = 1;
        // Nombre total d'image.
        $scope.totalPictures = 1;
        // Si on demande à confirmer une action.
        $scope.confirmAction = false;
        // Si on souhaite afficher tous les boutons d'actions ou limiter à certaines actions.
        $scope.minimalFooter = false;

        // On récupère l'indication pour savoir si on affiche toutes les photos ou uniquement celles d'un depot.
        var readTotalProject = $routeParams.readTotalProject === "true" ? true : false;
        // On récupère l'éventuel id de photo de départ.
        var idPictureStart = $routeParams.idPicture;
        if (idPictureStart) {
            $scope.minimalFooter = true;
        }

        // On charge le projet actuellement affiché.
        var projet = ProjetService.currentProjet;
        // Toutes les photos que l'on souhaite afficher.
        var pictures = [];
        var depots = [];
        if (readTotalProject) {
            depots = projet.depots;
        } else {
            var currentDepot = ProjetService.currentDepot;
            if (currentDepot) {
                depots = [currentDepot];
            }
        }
        for (var i = 0; i < depots.length; i++) {
            var depot = depots[i];
            for (var j = 0; j < depot.photos.length; j++) {
                pictures.push({depot: depot, photo: depot.photos[j]});
                if (idPictureStart == depot.photos[j].id) {
                    $scope.currentPositionPicture = pictures.length;
                }
            }
        }

        if (pictures.length) {
            $scope.totalPictures = pictures.length;
            $scope.pictures = pictures;
        } else {
            // Aucune photo, on supprime les dépôts vides.
            for (var i = 0; i < depots.length; i++) {
                projet.deleteDepot(depots[i]);
            }
            ProjetService.updateProjet(projet).then(function () {
                window.history.back();
            });
        }

        // Actions sur les boutons.
        $scope.takePicture = function () {
            ProjetService.currentPhoto = null;
            $location.path('/cameraChoix/');
        };

        // Démarre l'action pour supprimer la photo.
        $scope.startActionToDeletePicture = function() {
            $scope.confirmAction = true;
            currentAction = CURRENT_ACTION_DELETE_PICTURE;
        };

        // Démarre l'action pour délier une photo du dépot.
        $scope.startActionToUnlinkPicture = function() {
            $scope.confirmAction = true;
            currentAction = CURRENT_ACTION_UNLINK_PICTURE;
        };

        // Annule l'action en cours.
        $scope.cancelAction = function() {
            $scope.confirmAction = false;
        };

        // Effectue l'action.
        $scope.executeAction = function() {
            var promise;
            var depot = pictures[$scope.currentPositionPicture-1].depot;
            var photo = pictures[$scope.currentPositionPicture-1].photo;

            switch (currentAction) {
                case CURRENT_ACTION_DELETE_PICTURE:
                    promise = executeDelete(depot, photo);
                    break;
                case CURRENT_ACTION_UNLINK_PICTURE:
                    promise = executeUnlink(depot, photo);
                    break;
                default:
                    return;
            }

            promise.then(function () {
                if (depot.countPhotos() == 0) {
                    // Si le depot est vide, on le supprime.
                    projet.deleteDepot(depot);
                } else {
                    // On reset les bases64 du depot.
                    depot.resetBase64();
                }

                ProjetService.updateProjet(projet).then(function () {
                    if (!$scope.minimalFooter) {
                        $route.reload();
                    } else {
                        ProjetService.redirectToGallery();
                    }
                });
            });
        };

        // Lancé par la directive quand les images sont chargées. Permet de mettre en place le swipe entre images.
        $scope.$on("pictureReaderLoadFinish", function() {
            angular.element("#slider-picture").css("width", window.innerWidth+"px");
            window.pictureReaderSwipe = Swipe(document.getElementById("slider-picture"), {
                startSlide: $scope.currentPositionPicture-1,
                callback: function(index, elem) {
                    $scope.$apply(function() {
                        $scope.currentPositionPicture = index+1;
                    });
                }
            });
            angular.element("#slider-picture img").each(function(e) {
                new window.RTP.PinchZoom(this);
                angular.element(this).parent().css("margin-top", (angular.element(".content-cell").height() - angular.element(this).height()) / 2);
            });
        });

        /**
         * Effectue la suppression de l'image.
         *
         * @param {Depot} depot
         * @param {Photo} photo
         *
         * @return {Promise}
         */
        function executeDelete(depot, photo) {
            return new Promise(function (resolve) {
                GeneanetFile.deleteFileFromUrl(photo.nativeUrl).finally(function () {
                    depot.deletePhoto(photo);
                    resolve();
                });
            });
        }

        /**
         * Délie l'image.
         *
         * @param {Depot} depot
         * @param {Photo} photo
         *
         * @return {Promise}
         */
        function executeUnlink(depot, photo) {
            return new Promise(function (resolve) {
                // On retire la photo de l'ancien depot.
                depot.deletePhoto(photo);

                // On crée le nouveau depot et on ajoute la photo.
                var newDepot = projet.addDepot();
                newDepot.addPhoto(photo);

                resolve();
            });
        }
    }
];