'use strict';

var synchroController = [
    '$scope',
    '$location',
    '$timeout',
    '$route',
    '$routeParams',
    '$translate',
    'GeneanetDevice',
    'GeneanetOverlay',
    'GeneanetFile',
    'ConnexionService',
    'ProjetService',
    'ConfigurationService',
    'GeneanetLocalStorage',
    'urlsWebservice',
    function($scope, $location, $timeout, $route, $routeParams, $translate, GeneanetDevice, GeneanetOverlay, GeneanetFile, ConnexionService, ProjetService, ConfigurationService, GeneanetLocalStorage, urlsWebservice) {
        document.addEventListener('deviceready', GeneanetDevice.determineOrientationCurrentPage);

        $scope.back = function () {
            window.history.back();
        };

        $scope.noProjet = true;
        $scope.currentProjet = ProjetService.currentProjet;

        var urlGnn = encodeURI(urlsWebservice.galleryUrl + 'synchro/image');
        var projetsInfos = {};
        var deleteFiles = false;
        var synchroInProgress = false;
        var errorDuringUpload = false;

        // On génère un id unique pour l'application, si n'existe pas déjà, et on l'utilisera lors de la synchro.
        var uniqueId = (Math.random().toString(16)+'000000000').substr(2,8) +'-'+ new Date().getTime();
        ConfigurationService.add('id_application', uniqueId);

        // On vérifie qu'on est bien connecté.
        checkConnexion().then(function () {
            // On détermine les projets qu'il faut synchroniser.
            var projetsToList = ProjetService.currentProjet ? [ProjetService.currentProjet] : ProjetService.projets;
            for (var i = 0; i < projetsToList.length; i++) {
                if (projetsToList[i].hasPhotosToSynchro()) {
                    var nbPhotos = projetsToList[i].countPhotos(true);
                    var nbPhotosText = nbPhotos + ' ' + $translate.instant(nbPhotos <= 1 ? 'list_projectpage.picture' : 'list_projectpage.pictures');

                    projetsInfos[projetsToList[i].id] = {projet: projetsToList[i], nbPhotosText: nbPhotosText};
                }
            }

            // on affiche la liste des projets.
            $scope.$apply(function() {
                $scope.projetsInfos = projetsInfos;
                $scope.noProjet = Object.keys(projetsInfos).length > 0 ? false : true;
            });
        });

        // Action sur le bouton pour démarrer la synchro.
        $scope.runSynchro = function () {
            if (synchroInProgress) {
                console.error('Une synchro est déjà en cours.');
                return;
            }

            synchroInProgress = true;

            var idsProjectsChecked = [];
            angular.element('#synchro input[type="checkbox"]:checked').each(function() {
                idsProjectsChecked.push(angular.element(this).val());
            });

            if (idsProjectsChecked.length == 0) {
                GeneanetOverlay.openOverlay(
                    $translate.instant('alert.synchronize_project_no_selected'),
                    $translate.instant('alert.synchronize_need_project_selected')
                );
                synchroInProgress = false;
            } else {
                var projetsToSynchro = [];
                for (var i = 0; i < idsProjectsChecked.length; i++) {
                    projetsToSynchro.push(projetsInfos[idsProjectsChecked[i]].projet);
                }

                $scope.$on('overlayDialogResponse', function(event, data) {
                    deleteFiles = data.response;

                    // On démarre la synchro en commencant par les projets.
                    synchroProjets(projetsToSynchro)
                        // Si tout se passe bien, on lance la synchro des photos.
                        .then(function () {
                            synchroPhotos(projetsToSynchro);
                        });
                });

                // demande à l'utilisateur si on veut supprimer les photos une fois synchronisées ou pas.
                GeneanetOverlay.openDialog(
                    $translate.instant('dialog.delete_files'),
                    $translate.instant('dialog.yes'),
                    'btn btn-gnn-primary',
                    $translate.instant('dialog.no'),
                    'btn btn-gnn-secondary'
                );
            }
        };

        // Event se déclenchant à la fin de la synchro des images.
        $scope.$on('synchroPicturesFinished', function () {
            // La synchro est finie, on supprime les dépôts vides.
            var projetsUpdatedPromises = [];
            for (var projetId in projetsInfos) {
                var projet = projetsInfos[projetId].projet;
                var depotsDeleted = projet.deleteEmptyDepots();
                if (depotsDeleted) {
                    projetsUpdatedPromises.push(ProjetService.updateProjet(projet));
                }
            }
            Promise.all(projetsUpdatedPromises).then(function () {
                // Lorsque l'utilisateur ferme l'overlay de validation de synchro, on actualise sur la page.
                $scope.$on('GeneanetOverlayClose', function() {
                    $location.path('/synchro/true').replace();
                    $route.reload();
                });
                // Si une erreur non bloquante est survenue durant la synchro on notifie l'utilisateur.
                if (errorDuringUpload) {
                    GeneanetOverlay.openOverlay(
                        $translate.instant('alert.error_during_upload'),
                        $translate.instant('alert.error_during_upload_more')
                    );
                } else {
                    // si non, on indique juste que la synchro est finie.
                    GeneanetOverlay.openOverlay(
                        $translate.instant('alert.synchro_finished'),
                        $translate.instant('alert.synchro_finished_more')
                    );
                }
            });
        });

        /**
         * Vérifie la connexion à internet et au serveur.
         *
         * @return {Promise} On ne fait que du resolve. En cas d'echec, la méthode se débrouille à faire ce qu'il faut.
         */
        function checkConnexion() {
            return new Promise(function (resolve) {
                // check connexion
                if (!GeneanetDevice.accessInternetStarted()) {
                    $scope.$on('GeneanetOverlayClose', function() {
                        $scope.back();
                    });

                    GeneanetOverlay.openOverlay(
                        $translate.instant('alert.offline'),
                        $translate.instant('alert.cant_synchro')
                    );
                } else {
                    // check la connexion à GNN.
                    GeneanetOverlay.openSpinner($translate.instant('spinner.check_connexion'));
                    ConnexionService.isConnected()
                        .then(function () {
                            GeneanetOverlay.close();
                            if (!GeneanetDevice.wifiStarted() && !$routeParams.noAlertWifi) {
                                GeneanetOverlay.openOverlay(
                                    $translate.instant('alert.not_wifi'),
                                    $translate.instant('alert.wifi_better')
                                );
                            }
                            resolve();
                        })
                        .catch(redirectNoConnexion);
                }
            });
        }

        /**
         * Permet d'ouvrir l'overlay pour signaler que l'on doit avoir internet.
         */
        function openOverlayRedirectConnexion() {
            $scope.$on('GeneanetOverlayClose', function() {
                redirectNoConnexion();
            });
            GeneanetOverlay.openOverlay(
                $translate.instant('alert.lost_connexion'),
                $translate.instant('alert.lost_connexion_more')
            );
        }

        /**
         * Permet de rediriger si non connecté.
         */
        function redirectNoConnexion() {
            ConnexionService.setRedirect('/synchro/');
            $timeout(function() {
                $scope.$apply(function() {
                    $location.path('/connexion/').replace();
                });
            }, 11);
        }

        /**
         * Permet de synchroniser les projets séléctionnés.
         *
         * @param {Array} projetsToSynchro
         *
         * @return {Promise}
         */
        function synchroProjets(projetsToSynchro) {
            // On démarre une progressbar.
            GeneanetOverlay.openProgressBar($translate.instant('progress_bar.synchro_project'), 0, projetsToSynchro.length);

            // Liste des promises suivant la synchro des projets.
            var promises = [];
            for (var i = 0; i < projetsToSynchro.length; i++) {
                var projet = projetsToSynchro[i];

                // on regarde s'il s'agit d'une édition ou d'un nouveau projet.
                var isEdit = false;
                if (projet.isPrivate() && !projet.isSync) {
                    isEdit = true;
                }

                var params = {};
                if (projet.idProjetGnn != null) {
                    params.id = projet.idProjetGnn;
                }
                if (isEdit) {
                    params.place = projet.place;
                    params.nom_projet = projet.nom;
                    params.actif = 0;
                    params.type_projet = projet.type;

                    var geoCurrent = projet.getGeo();
                    if (geoCurrent.length == 2 && !isNaN(geoCurrent[0]) && !isNaN(geoCurrent[1])) {
                        params.lat = geoCurrent[0];
                        params.lon = geoCurrent[1];
                    } else {
                        params.country = geoCurrent[0];
                        if (geoCurrent.length > 1) {
                            params.region = geoCurrent[1];
                            if (geoCurrent.length > 2) {
                                params.subregion = geoCurrent[2];
                            }
                        }
                    }
                }

                promises.push(synchroProjet(projet, params));
            }

            var synchroPromisesAll = Promise.all(promises);
            synchroPromisesAll
                .then(function () {
                    // Normalement, tout est synchro, on ferme l'overlay.
                    if (GeneanetOverlay.progressBarIsFinished()) {
                        GeneanetOverlay.close();
                    }
                })
                .catch(function (err) {
                    if (err == 'no_connected') {
                        openOverlayRedirectConnexion();
                    } else {
                        GeneanetOverlay.openOverlay(
                            $translate.instant('alert.fail_connect_gnn'),
                            $translate.instant('alert.fail_connect_gnn_more')
                        );
                    }
                });

            return synchroPromisesAll;
        }

        /**
         * Lance la synchro d'un projet.
         *
         * @param {Projet} projet
         * @param {Object} params
         *
         * @return {Promise}
         */
        function synchroProjet(projet, params) {
            return new Promise(function (resolve, reject) {
                $.post(urlsWebservice.galleryUrl+'projects/add', params)
                    .success(function(data) {
                        if (data.success) {
                            if (data.project) {
                                // met a jour l'id du projet, le propriétaire et on indique que le projet est synchronisé.
                                projet.idProjetGnn = data.project.id;
                                projet.proprietaire = data.project.sourcename_proprietaire;
                                projet.isSync = true;

                                // incrémente de 1 le nombre de projets au quel l'utilisateur a participé.
                                var projectsPublished = GeneanetLocalStorage.getItemWithDefaultValue('projectsPublished', [], 'configuration');
                                if (projectsPublished.indexOf(projet.idProjetGnn) === -1) {
                                    projectsPublished.push(projet.idProjetGnn);
                                }
                                GeneanetLocalStorage.setItem('projectsPublished', projectsPublished, 'configuration');

                                // On update le projet.
                                ProjetService.updateProjet(projet).then(resolve);
                            } else {
                                // Ce n'est pas grave, on continue quand même.
                                console.error(data);
                                resolve();
                            }

                            // on avance la progress bar.
                            GeneanetOverlay.advance();
                        } else if (data.error == 'no_connected') {
                            // si nn connecté, on redirige à la page de connexion.
                            reject('no_connected');
                        } else {
                            reject();
                        }
                    })
                    .error(reject);
            });
        }

        /**
         * Lance la synchro des photos.
         *
         * @param {Array} projetsToSynchro
         */
        function synchroPhotos(projetsToSynchro) {
            // On récupère l'ensemble des photos a synchroniser.
            var photosToSynchro = [];
            var totalPhotos = 0;
            for (var i = 0; i < projetsToSynchro.length; i++) {
                var photos = projetsToSynchro[i].getPhotosToSynchro();
                totalPhotos += photos.length;
                photosToSynchro.push(photos);
            }

            // On ouvre l'overlay + progressbar de progression.
            var divProgressBarFile = angular.element('<div>')
                .attr('id', 'progress-bar-file')
                .html('<div></div>');
            var divDescription = angular.element('<div>')
                .text($translate.instant('progress_bar.synchro_picture'));

            var buttonStopSynchro = angular.element('<button>')
                .addClass('btn btn-gnn-secondary')
                .click(function() {
                    angular.element(this).text($translate.instant('progress_bar.synchro_stop_progress'))
                        .attr('disabled', 'disabled');
                    synchroInProgress = false;
                })
                .text($translate.instant('progress_bar.synchro_stop'));
            var divButton = angular.element('<div>')
                .addClass('text-center')
                .append(buttonStopSynchro);

            var htmlOverlay = angular.element('<div>')
                .append(divProgressBarFile)
                .append(divDescription)
                .append(divButton);
            GeneanetOverlay.openProgressBar(htmlOverlay, 0, totalPhotos);

            synchroPhoto(photosToSynchro);
        }

        /**
         * Publie une photo et passe à la suivante.
         *
         * @param {Array} photosToSynchro
         */
        function synchroPhoto(photosToSynchro) {
            if (photosToSynchro.length == 0 || !synchroInProgress) {
                // On a fini ou bien la synchro a été stoppée.
                $scope.$broadcast('synchroPicturesFinished', {});
                return;
            }

            var photoToSynchro = photosToSynchro[0].shift();
            var projet = photoToSynchro.projet;
            var depot = photoToSynchro.depot;
            var photo = photoToSynchro.photo;

            // Permet de lancer la synchro de la photo suivante.
            var nextPhoto = function () {
                // on avance la progress bar & on met à jour le nombre de photos synchronisées par l'utilisateur.
                GeneanetOverlay.advance();
                ConfigurationService.edit('picturesPublished', parseInt(ConfigurationService.get('picturesPublished'))+1);

                if (photosToSynchro[0].length == 0) {
                    // On a fini les photos du projet courant. On retire l'array vide.
                    endSynchro(projet.idProjetGnn);
                    photosToSynchro.shift();
                }

                // On passe à la suite.
                synchroPhoto(photosToSynchro);
            };

            // Indique au serveur qu'on démarre une synchro sur le projet (ou récupère l'id de synchro si déjà démarrée).
            var idSynchro = startSynchro(projet.idProjetGnn);
            if (idSynchro === false) {
                GeneanetOverlay.openOverlay(
                    $translate.instant('alert.fail_synchro'),
                    $translate.instant('alert.fail_synchro_more')
                );
                synchroInProgress = false;
                return;
            }

            // On prépare l'envoi du fichier.
            var ft = createFileTransfer();
            // On génère les options pour l'envoi du fichier.
            var options = createFileUploadOptions(idSynchro, depot, photo);
            // On envoie le fichier.
            var promise = new Promise(function (resolve, reject) {
                // lance l'upload du fichier.
                ft.upload(photo.nativeUrl, urlGnn, resolve, reject, options);
            });
            promise.then(function (response) {
                // L'envoi a réussi.
                var data = JSON.parse(response.response);
                if (!data.success && data.error == 'no_connected') {
                    openOverlayRedirectConnexion();
                } else if (data.success) {
                    // on supprime le fichier & la ligne en BDD si demandé au début de la synchro.
                    if (deleteFiles) {
                        GeneanetFile.deleteFileFromUrl(photo.nativeUrl).then(function () {
                            depot.deletePhoto(photo);
                            ProjetService.updateProjet(projet).then(nextPhoto);
                        });
                    } else {
                        // on passe la photo en état de synchronisée.
                        photo.isSync = true;
                        ProjetService.updateProjet(projet).then(nextPhoto);
                    }
                } else {
                    // pas réussi mais pas à cause de non connection, on passe au suivant et on notifiera l'utilisateur à la fin.
                    nextPhoto();
                    errorDuringUpload = true;
                }
            }).catch(function (error) {
                // L'envoi a échoué.
                if (error.code === FileTransferError.FILE_NOT_FOUND_ERR) {
                    // Le fichier n'a pas été trouvé, on supprime la photo.
                    depot.deletePhoto(photo);
                    ProjetService.updateProjet(projet).then(nextPhoto);
                } else {
                    // gnn HS ou autre soucis
                    $scope.$on('GeneanetOverlayClose', function() {
                        $route.reload();
                    });
                    GeneanetOverlay.openOverlay(
                        $translate.instant('alert.fail_connect_gnn'),
                        $translate.instant('alert.fail_connect_gnn_more')
                    );
                }
            });
        }

        /**
         * Prépare un nouvel envoi de fichier.
         *
         * @return {FileTransfer}
         */
        function createFileTransfer() {
            // On prépare l'envoi du fichier.
            var ft = new FileTransfer();
            // On remet à 0 la progressBar d'envoi du fichier que l'on fera avancer pendant l'envoi du fichier..
            if (angular.element('#progress-bar-file')) {
                angular.element('#progress-bar-file > div').css('width', '0%');
            }
            ft.onprogress = function(progressEvent) {
                if (progressEvent.lengthComputable) {
                    if (angular.element('#progress-bar-file')) {
                        var pourcent = progressEvent.loaded / progressEvent.total * 100;
                        angular.element('#progress-bar-file > div').css('width', pourcent+'%');
                    }
                }
            };

            return ft;
        }

        /**
         * Prépare les options pour l'envoi du fichier.
         *
         * @param {integer} idSynchro
         * @param {Depot} depot
         * @param {Photo} photo
         *
         * @return {FileUploadOptions}
         */
        function createFileUploadOptions(idSynchro, depot, photo) {
            var options = new FileUploadOptions();
            var fileUrl = photo.nativeUrl;
            options.fileKey = 'file';
            options.mimeType = 'image/jpeg';
            options.fileName = fileUrl.substr(fileUrl.lastIndexOf('/')+1);
            options.params = {
                id_synchro_project: idSynchro,
                id_depot: depot.id,
                id_section: depot.sectionId,
                id_photo: photo.id
            };

            return options;
        }

        /**
         * Permet de démarrer une synchronisation sur un projet.
         *
         * @param  {integer} idProject id du projet visé.
         *
         * @return {integer} Retourne l'id de synchro.
         */
        function startSynchro(idProject) {
            return changeStepSynchro(idProject, 0);
        }

        /**
         * Permet de stopper une synchronisation sur un projet.
         *
         * @param  {integer} idProject id du projet visé.
         *
         * @return {integer} Retourne l'id de synchro.
         */
        function endSynchro(idProject) {
            return changeStepSynchro(idProject, 1);
        }

        /**
         * Permet de changer l'état d'une synchronisation avec le serveur.
         *
         * @param {integer} idProject Id du projet visé.
         * @param {integer} finish    1 si fini, 0 si non.
         *
         * @return {integer} Retourne l'id de synchro.
         */
        function changeStepSynchro(idProject, finish) {
            var idSynchro = false;

            var params = {
                id: idProject,
                finish: finish,
                id_application: ConfigurationService.get('id_application')
            };

            // envoye à GNN.
            $.ajax({
                type: 'POST',
                async: false,
                data: params,
                url: urlsWebservice.galleryUrl + 'synchro/project',
                success: function(data) {
                    if (!data.success && data.error == 'no_connected') {
                        openOverlayRedirectConnexion();
                    } else if (data.success && data.id_synchro_project) {
                        idSynchro = data.id_synchro_project;
                    }
                },
                error: function() {
                    GeneanetOverlay.openOverlay(
                        $translate.instant('alert.fail_connect_gnn'),
                        $translate.instant('alert.fail_connect_gnn_more')
                    );
                }
            });

            return idSynchro;
        }
    }
]