'use strict';

/**
 *  Service permettant de gérer la configuration de l'application.
 */
var connexionService = [
    "$rootScope",
    "$location",
    "$timeout",
    "urlsWebservice",
    function($rootScope, $location, $timeout, urlsWebservice) {

        var redirect = null;

        this.getRedirect = function() {
            return redirect;
        }

        this.setRedirect = function(newRedirect) {
            redirect = newRedirect;
        }

        this.redirect = function() {
            if (!redirect) {
                redirect = "/";
            }

            $rootScope.$apply(function() {
                $timeout(function() {
                    $rootScope.$apply(function() {
                        $location.path(redirect).replace();
                    });
                }, 11);
            });
        }

        /**
         * Verifie que l'utilisateur est connecté.
         *
         * @return {Promise}
         */
        this.isConnected = function() {
            return execConnexion();
        };

        /**
         * Permet de connecter un utilisateur
         *
         * @param {string} login     Login GNN
         * @param {string} password  Mot de passe GNN
         *
         * @return {Promise}
         */
        this.connect = function(login, password) {
            return execConnexion({login: login, password: password});
        };

        /**
         * Permet de se déconnecter.
         *
         * @return {Promise}
         */
        this.disconnect = function() {
            return execConnexion({nologin: 1});
        };

        /**
         * Démarre la connexion.
         *
         * @param {Object} params
         *
         * @return {Promise}
         */
        function execConnexion(params) {
            if (!params) {
                params = {};
            }

            return new Promise(function (resolve, reject) {
                $.post(urlsWebservice.connexionUrl, params)
                    .success(function(data) {
                        if (data == 1) {
                            resolve();
                        } else {
                            reject();
                        }
                    })
                    .error(function(data) {
                        reject();
                    });
            });
        }
    }
]