'use strict';

/**
 * Service GeneanetFile
 * Permet la manipulation de fichier et de dossier
 *     ** Acceder à (ou créer) un fichier **
 *     ** Copier un fichier **
 *     ** Supprimer un ficher **
 */
angular.module('GeneanetService').service('GeneanetFile', [
    function() {
        var currentInstanceObject = this;

        var echecFichier = function (error) {
            console.log("File error : " + error.message);
        };

        var echecDossier = function (error) {
            console.log("Directory error : " + error.message);
        };

        this.copierFichier = function (fileEntry, fnSuccess, newName, destination) {
            fileEntry.moveTo(destination, newName, fnSuccess, echecFichier);
        }

        this.supprimerFichier = function (fileEntry, fnSuccess) {
            fileEntry.remove(fnSuccess, echecFichier);
        }

        var supprimerVide = function (tab){
            for (var i in tab) {
                if (tab[i].trim() == '') {
                    tab.splice(i, 1);
                }
            }
            return tab;
        }

        this.obtenirFichier = function (pathComplet, fnGetSuccess ) {
            var getFile = function ( fileSystem ) {
                fileSystem.root.getFile( pathComplet, null, fnGetSuccess, echecFichier );
            }

            window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, getFile, echecFichier);
        }

        var obtenirDossier = function ( pathComplet, fileSystem, fileEntry, fnGetSuccess){
            var dir, sousPath, tabDir;
            tabDir = pathComplet.split('/');
            tabDir = supprimerVide(tabDir);
            if(tabDir.length<=0){
                console.log("Error path is empty");
            }
            else if(tabDir[0]==""){
                tabDir.shift();
                sousPath = tabDir.join('/');
                obtenirDossier (sousPath, fileSystem, fileEntry, fnGetSuccess);
            }else{
                dir = tabDir[0];
                fileSystem.getDirectory(
                    dir,
                    {
                        create : true,
                        exclusive: false
                    },
                    function ( dataDir) {
                        if (tabDir.length>1){
                            tabDir.shift();
                            sousPath = tabDir.join('/');
                            obtenirDossier (sousPath, dataDir, fileEntry, fnGetSuccess);
                        }else{
                            if (fnGetSuccess != undefined) {
                                fnGetSuccess(dataDir);
                            }
                        }
                    },
                    echecDossier
                );
            }
        };

        this.accesDossier = function(pathComplet, fileSystem, fileEntry, fnGetSuccess) {
            obtenirDossier(pathComplet, fileSystem, fileEntry, fnGetSuccess);
        }

        /**
         * Récupère l'accès à un fichier depuis son url.
         *
         * @param {string} url
         *
         * @return {Promise}
         */
        this.getFileFromUrl = function (url) {
            return new Promise(function (resolve, reject) {
                window.resolveLocalFileSystemURL(url, resolve, function (codeError) {
                    console.error("Access file failed : "+url);
                    console.log(codeError);
                    reject(codeError);
                });

            });
        };

        /**
         * Supprime un fichier depuis son url.
         *
         * @param {string} url
         *
         * @return {Promise}
         */
        this.deleteFileFromUrl = function (url) {
            return new Promise(function (resolve, reject) {
                currentInstanceObject.getFileFromUrl(url)
                    .then(function (fileEntry) {
                        // On a récupéré le fichier.
                        fileEntry.remove(resolve, function (codeError) {
                            console.error("Delete file failed : "+url);
                            console.log(codeError);
                            reject(codeError);
                        });
                    })
                    .catch(function () {
                        // Le fichier n'est pas dispo, on considère qu'il n'existe plus et donc que le delete est ok.
                        resolve();
                    });
            });
        };

        /**
         * Permet de récupérer le dossier enfant de l'entity.
         *
         * @param  {object}  entity          Entitée dont on veut le dossier enfant.
         * @param  {string}  nameDirectory   Nom du dossier que l'on désire.
         * @param  {boolean} createIfNoExist Permet de déterminer si l'on veut créer le dossier si non existant.
         *
         * @return {Promise}
         */
        this.getDirectory = function(entity, nameDirectory, createIfNoExist) {
            if (typeof createIfNoExist == "undefined") {
                createIfNoExist = false;
            }

            return new Promise(function (resolve, reject) {
                entity.getDirectory(
                    nameDirectory,
                    {
                        create: createIfNoExist,
                        exclusive: false
                    },
                    resolve,
                    function (codeError) {
                        console.log("Error to get directory of entity, error : '" + codeError + "'");
                        reject(codeError);
                    }
                );
            });
        };

        /**
         * Permet de récupérer le parent d'un FileEntry ou DirectoryEntry.
         *
         * @param {Object} entity Contient l'objet dont on veut le dossier parent.
         *
         * @return {Promise}
         */
        this.entityGetParent = function(entity) {
            return new Promise(function (resolve, reject) {
                entity.getParent(
                    resolve,
                    function (codeError) {
                        console.log("Error to get parent of entity, error : '"+codeError+"'");
                        reject(codeError);
                    }
                );
            });
        }

        /**
         * Permet de récupérer l'extention d'un fichier depuis son nom.
         *
         * @param  {string} nameFile Nom du fichier.
         *
         * @return {string | boolean} Renvoie l'extention du fichier sous forme de
         *                            chaine de caractère ou false si rien récupérer.
         */
        this.getExtensionFile = function(nameFile) {
            var regExtension = new RegExp("[A-Za-z0-9]*$");
            var result = nameFile.match(regExtension);
            if (result.length > 0) {
                return result[0];
            } else {
                return false;
            }
        }
    }
]);