'use strict';

/**
 * Service qui s'occupe de la localisation
 */
angular.module('GeneanetService').service('GeneanetGeolocation',[
    "$rootScope",
    function ($rootScope) {
        /**
         * Permet de connaitre la position actuelle de l'utilisateur et retourne l'info via une promise.
         *
         * @param  {integer} timeCache          Temps maximum en milliseconde de valider du cache pour la position.
         * @param  {integer} timeout            Temps maximum en milliseconde pour tenter de déterminer la position.
         * @param  {boolean} enableHighAccuracy A true : permet d'exploiter la meilleur méthode pour
         *                                      localiser, false seulement via réseau.
         *
         * @return {Promise}
         */
        this.getCurrentPosition = function (timeCache, timeout, enableHighAccuracy) {
            var options = {
                maximumAge: timeCache ? timeCache : null,
                timeout: timeout ? timeout : null,
                enableHighAccuracy: enableHighAccuracy ? enableHighAccuracy : false
            };

            return new Promise(function (resolve, reject) {
                navigator.geolocation.getCurrentPosition(
                    function (position) {
                        resolve(position);
                    },
                    function (positionError) {
                        reject(positionError);
                    },
                    options
                );
            });
        };
    }
]);